#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from typing import List, Optional
import os
import re

from PyQt5.QtCore import QTemporaryDir

from astra_csp_cryptopro.zip_archiver import ZipArchiver
from astra_csp_cryptopro.utils import Utils
from astra_csp_cryptopro.unpack_archive_thread import UnpackArchiveThread


class Container:
    def __init__(self, filepath: str, tmpdir: str, external: bool = False) -> None:
        self.__filepath = filepath
        self.__tmpdir = tmpdir
        self.__is_external = external
        self.__mountdir = QTemporaryDir(self.__tmpdir)
        self.__mountdir.setAutoRemove(False)

    @property
    def path(self) -> str:
        return self.__filepath

    @property
    def is_external(self) -> bool:
        return self.__is_external

    @property
    def mount_dir(self) -> Optional[QTemporaryDir]:
        return self.__mountdir

    @property
    def name(self) -> str:
        return os.path.basename(self.__filepath)

    def files(self) -> List[str]:
        if self.__is_external:
            for dir_path, directory_names, filenames in os.walk(self.__filepath):
                return filenames
        else:
            archive = ZipArchiver(self.__filepath, self.__mountdir.path())
            return [file_info.name for file_info in archive.info().files_info]

    def sources_path(self) -> str:
        if self.is_external:
            return self.path
        else:
            return self.mount_dir.path()

    def create_unpack_thread(self, filenames: List[str] = None) -> UnpackArchiveThread:
        archive = ZipArchiver(self.__filepath, self.__mountdir.path())
        return UnpackArchiveThread(archive=archive, filenames=filenames)

    def unpacked(self) -> bool:
        if not self.__mountdir:
            return False

        if not self.__mountdir.isValid():
            return False

        archive = ZipArchiver(self.__filepath, self.__mountdir.path())
        return archive.validate_unpacked_files()

    def update(self) -> None:
        if self.__mountdir:
            archive = ZipArchiver(self.__filepath, self.__mountdir.path())
            archive.update()
            self.__mountdir.remove()

    def save(self, files: List[str], update: bool = False) -> None:
        tmpdir = QTemporaryDir(self.__tmpdir)

        for file in files:
            if file:
                temp_symlink_path = tmpdir.path() + '/' + os.path.basename(file) + '.temp'
                os.symlink(file, temp_symlink_path)
                os.rename(temp_symlink_path, tmpdir.path() + '/' + os.path.basename(file))

        # bsdtar-created archives produces issues with archivemount on Smolensk 1.5
        # fly-csp on 1.6 and 1.5 should have exactly same implementation to create compatible containers
        # archive = BsdTarArchiver(self.__filepath, tmpdir.path())
        archive = ZipArchiver(self.__filepath, tmpdir.path())
        archive.create(update)

    def contains(self, filename: str) -> bool:
        if self.__is_external:
            return Utils.file_is_exists(self.__filepath + '/' + filename)
        else:
            archive = ZipArchiver(self.__filepath, self.__mountdir.path())
            return archive.info().file_exists(filename)

    @classmethod
    def is_container(cls, filename: str) -> bool:
        return filename.endswith('.signed.zip')

    @classmethod
    def is_external_container(cls, filename: str) -> bool:
        dirname = os.path.dirname(filename)
        math = re.match(r'/run/user/\d+/fly-fm-vfs/archives/[0-9a-fA-F]+', dirname)
        return bool(math)
